﻿using System.Collections.Immutable;
using System.Collections.ObjectModel;

namespace Collections_06
{
    /// <summary>
    /// Based on demo created by Filip Opaleny and Milan Mikus.
    /// </summary>
    class Program
    {
        /*
            Pripomenuti ruznych typu kolekci:

            negenericke:
            ArrayList, Hashtable, BitArray, ...

            genericke:
            List, LinkedList, SortedList Dictionary,
            Stack, Queue, HashSet, SortedSet, ...
        */

        static void Main(string[] args)
        {
            // zadani samostane prace:
            //
            // I. implementace tridy CustomerManager
            //
            // II. overeni zda program po spusteni vypisuje
            // nize uvedena testovaci data

            Solution.Test();

            // Ukazka readonly kolekci
            ReadonlyDemo();
            ImmutableDemo();

        }

        private static void ReadonlyDemo()
        {
            var list = new List<MuteableClass> { new MuteableClass() };

            var readonlyList = new ReadOnlyCollection<MuteableClass>(list);

            Console.WriteLine(ToString(readonlyList));
            //can't be changed from class property
            //readonlyList.ReadOnlyList.Add();

            //inner properties are not protected!
            readonlyList[0].NumberOne = "Saru";
            Console.WriteLine(ToString(readonlyList));

            //this of course changes the list
            list.Add(new MuteableClass() { NumberOne = "Una" });
            Console.WriteLine(ToString(readonlyList));

            Console.ReadKey();
        }

        private static void ImmutableDemo()
        {
            //creating them
            var immutable1 = ImmutableList.Create(new[] { "1", "2", "3" });
            var immutable2 = ImmutableList.Create("1", "2", "3");

            var ordinaryList = new List<string> { "1", "2", "3" };
            var immutable3 = ordinaryList.ToImmutableList();

            //original won't change
            Console.WriteLine(ToString(immutable1.Add("4")));
            Console.WriteLine(ToString(immutable1.Add("5")));

            var newImmutable = immutable1.AddRange(new[] { "4", "5", "6" });
            Console.WriteLine(ToString(newImmutable));

            Console.ReadKey();

            //How about inner properties?
            var immutableInner = ImmutableList.Create(new MuteableClass());
            immutableInner[0].NumberOne = "Saru";
            Console.WriteLine(ToString(immutableInner));

            Console.ReadKey();
        }

        private static string ToString(IEnumerable<object> enumerable) =>
            string.Join(",", enumerable);
    }
}
